#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ---------------------------------------------------------------------
# Read and Set  ATTEN PPS30005S
# ---------------------------------------------------------------------
# Copyright (c) 2014 boxtec internet appliances <boxtec@boxtec.ch>
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
# THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# --------------------------------------------------------------------------
#  $Id: pps3005ctl.py,v 1.3 2014-04-28 19:03:44 obiwan Exp $
# --------------------------------------------------------------------------
# ###########################################################
# TODO:
# NOTES:
# ###########################################################


"""
__author__  = "$Author: cschnee $ (cschnee@boxtec.ch)"
__appname__ = "pps3005ctl.py"
__version__ = "$Revision: 1.3 $"
__date__    = "$Date: 2014-04-28 19:03:44 $"
__license__ = "BSD"
"""

################
# IMPORTS
################
import serial, time, sys, signal, argparse

################
# CONSTANTS
################
std_msg = [0xAA, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, \
          0x00, 0x00, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x68]
ONOFFMAP = {0: 'OFF', 1: 'ON'}          

################
# METHODS / FUNCTIONS
################
def open_ser(port='/dev/ttyUSB0',baudrate=9600,timeout=3):
  """Returns an open com port."""
  return serial.Serial(port=port,baudrate=baudrate,timeout=timeout)

def close_ser(ser):
  """Close serial port and ev. do some cleanup."""
  ser.close()

def send_buffer(ser, buffer_lst):
  """Send buffer to device all in one packet."""
  msg = ''
  for c in buffer_lst:
    msg += chr(c)
  ser.write(msg)
  
def read_buffer(ser):  
  """Reads data from serial port and returns it."""
  buf = ser.read(24)
  current = (ord(buf[4])*256 + ord(buf[5])) / 1000.0
  voltage = (ord(buf[2])*256 + ord(buf[3])) / 100.0
  if args.quiet:
    print "%s %s %s" %(voltage, current, ONOFFMAP[std_msg[15]])
  else:
    print "Voltage:", voltage
    print "Current:", current
    print "Output: ", ONOFFMAP[std_msg[15]]
  
def myprint(line):
  """Prints output depending on verbosity"""
  if not args.quiet:
    print line
      
def shutdown(signum, frame):
    print "\nExiting..."
    #cleanup what you might need
    sys.exit(0)
 
################
# SETTINGS
################
def set_voltage(voltage):
  """Sets the voltage."""
  v = int(float(voltage) * 100)
  std_msg[2] = v / 256
  std_msg[3] = v % 256
  
def set_current(current):
  """Sets the current."""
  i = int(float(current) * 1000)
  std_msg[4] = i / 256
  std_msg[5] = i % 256

def set_output(output):
  """Sets output on/off."""
  std_msg[15] = int(output)

################
# MAIN
################
if __name__ == '__main__':
  signal.signal(signal.SIGINT, shutdown)      
  parser = argparse.ArgumentParser()
  parser.add_argument("-v", "--voltage", help="Set voltage to [0-30]V", type=float)
  parser.add_argument("-i", "--current", help="Set current to [0-5]A", type=float)
  parser.add_argument("-o", "--output", help="Switch output to ON/OFF [0|1]", choices=[0, 1], type=int)
  parser.add_argument("-p", "--port", help="Set serial port to: [COM0-99|/dev/ttyUSB0-99]", required=True)
  parser.add_argument("-q", "--quiet", help="Don't be verbose", action='store_true')
  parser.add_argument("-l", "--loop", help="Constantly read and output values from device", action='store_true')
  args = parser.parse_args()

  if args.port:
    myprint("Setting serial port to %s" %args.port)
    my_sport = args.port
  if args.voltage:
    myprint("Setting voltage to %s" %args.voltage)
    set_voltage(args.voltage)
  if args.current:
    myprint("Setting current to %s" %args.current)
    set_current(args.current)
  if args.output:
    myprint("Switching output to %s" %args.output)
    set_output(args.output)

  try:
    sp = open_ser(port=my_sport, baudrate=9600, timeout=5)
  except NameError, err:
    print "Error: -p defines not a valid serial port!" 
    sys.exit(2)

  # experimental OVP/OCP:
  #std_msg[18] = 1
  if args.loop:
    while ( 1 ): 
      send_buffer(sp, std_msg)
      read_buffer(sp)
  else:
    # do an empty reading round to let the output settle a bit
    send_buffer(sp, std_msg)
    sp.read(24)
    send_buffer(sp, std_msg)
    read_buffer(sp)
    
  close_ser(sp)
  